/*
 * Copyright (c) 2005 Martin Decky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup ppc32
 * @{
 */
/** @file
 */

#ifndef KERN_ppc32_CPU_H_
#define KERN_ppc32_CPU_H_

/* MSR bits */
#define MSR_DR  (1 << 4)
#define MSR_IR  (1 << 5)
#define MSR_PR  (1 << 14)
#define MSR_EE  (1 << 15)

/* HID0 bits */
#define HID0_STEN  (1 << 24)
#define HID0_ICE   (1 << 15)
#define HID0_DCE   (1 << 14)
#define HID0_ICFI  (1 << 11)
#define HID0_DCI   (1 << 10)

#ifndef __ASM__

#include <typedefs.h>
#include <trace.h>

typedef struct {
	uint16_t version;
	uint16_t revision;
} __attribute__ ((packed)) cpu_arch_t;

NO_TRACE static inline void cpu_version(cpu_arch_t *info)
{
	asm volatile (
		"mfpvr %[cpu_info]\n"
		: [cpu_info] "=r" (*info)
	);
}

#endif /* __ASM__ */

#endif

/** @}
 */
