/*
 * Copyright (c) 2006 Ondrej Palkovsky
 * Copyright (c) 2008 Martin Decky
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @defgroup msimfb MSIM text console
 * @brief HelenOS MSIM text console.
 * @ingroup fbs
 * @{
 */
/** @file
 */

#include <async.h>
#include <libc.h>
#include <sysinfo.h>
#include <as.h>
#include <ddi.h>
#include <errno.h>

#include "serial_console.h"
#include "msim.h"

#define WIDTH 80
#define HEIGHT 24

static char *virt_addr;

static void msim_putc(const char c)
{
	*virt_addr = c;
}

int msim_init(void)
{
	sysarg_t phys_addr;
	if (sysinfo_get_value("fb.address.physical", &phys_addr) != EOK)
		return -1;
	
	virt_addr = (char *) as_get_mappable_page(1);
	
	if (physmem_map((void *) phys_addr, virt_addr, 1,
	    AS_AREA_READ | AS_AREA_WRITE) != 0)
		return -1;
	
	serial_console_init(msim_putc, WIDTH, HEIGHT);
	
	async_set_client_connection(serial_client_connection);
	return 0;
}

/** @}
 */
